@@ -1,6 +1,6 @@
 -module(megaco_text_mini_parser).
 -export([parse/1, parse_and_scan/1, format_error/1]).
--file("megaco_text_mini_parser.yrl", 391).
+-file("megaco_text_mini_parser.yrl", 392).
 
 %% The following directive is needed for (significantly) faster compilation
 %% of the generated .erl file by the HiPE compiler.  Please do not remove.
@@ -9,22 +9,23 @@
 -include("megaco_text_mini_parser.hrl").
 
 
--file("/net/isildur/ldisk/daily_build/17_prebuild_opu_o.2014-12-09_21/otp_src_17/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
+-file("/net/isildur/ldisk/daily_build/20_prebuild_opu_o.2017-09-25_21/otp_src_20/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2013. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
 %%
-%% The contents of this file are subject to the Erlang Public License,
-%% Version 1.1, (the "License"); you may not use this file except in
-%% compliance with the License. You should have received a copy of the
-%% Erlang Public License along with this software. If not, it can be
-%% retrieved online at http://www.erlang.org/.
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and limitations
-%% under the License.
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
 %%
 %% %CopyrightEnd%
 %%
@@ -136,21 +137,10 @@ yecc_end(Line) ->
     {'$end', Line}.
 
 yecctoken_end_location(Token) ->
-    try
-        {text, Str} = erl_scan:token_info(Token, text),
-        {line, Line} = erl_scan:token_info(Token, line),
-        Parts = re:split(Str, "\n"),
-        Dline = length(Parts) - 1,
-        Yline = Line + Dline,
-        case erl_scan:token_info(Token, column) of
-            {column, Column} ->
-                Col = byte_size(lists:last(Parts)),
-                {Yline, Col + if Dline =:= 0 -> Column; true -> 1 end};
-            undefined ->
-                Yline
-        end
-    catch _:_ ->
-        yecctoken_location(Token)
+    try erl_anno:end_location(element(2, Token)) of
+        undefined -> yecctoken_location(Token);
+        Loc -> Loc
+    catch _:_ -> yecctoken_location(Token)
     end.
 
 -compile({nowarn_unused_function, yeccerror/1}).
@@ -161,40 +151,40 @@ yeccerror(Token) ->
 
 -compile({nowarn_unused_function, yecctoken_to_string/1}).
 yecctoken_to_string(Token) ->
-    case catch erl_scan:token_info(Token, text) of
-        {text, Txt} -> Txt;
-        _ -> yecctoken2string(Token)
+    try erl_scan:text(Token) of
+        undefined -> yecctoken2string(Token);
+        Txt -> Txt
+    catch _:_ -> yecctoken2string(Token)
     end.
 
 yecctoken_location(Token) ->
-    case catch erl_scan:token_info(Token, location) of
-        {location, Loc} -> Loc;
-        _ -> element(2, Token)
+    try erl_scan:location(Token)
+    catch _:_ -> element(2, Token)
     end.
 
 -compile({nowarn_unused_function, yecctoken2string/1}).
-yecctoken2string({atom, _, A}) -> io_lib:write(A);
+yecctoken2string({atom, _, A}) -> io_lib:write_atom(A);
 yecctoken2string({integer,_,N}) -> io_lib:write(N);
 yecctoken2string({float,_,F}) -> io_lib:write(F);
 yecctoken2string({char,_,C}) -> io_lib:write_char(C);
 yecctoken2string({var,_,V}) -> io_lib:format("~s", [V]);
 yecctoken2string({string,_,S}) -> io_lib:write_string(S);
 yecctoken2string({reserved_symbol, _, A}) -> io_lib:write(A);
-yecctoken2string({_Cat, _, Val}) -> io_lib:format("~p",[Val]);
+yecctoken2string({_Cat, _, Val}) -> io_lib:format("~tp", [Val]);
 yecctoken2string({dot, _}) -> "'.'";
-yecctoken2string({'$end', _}) ->
-    [];
+yecctoken2string({'$end', _}) -> [];
 yecctoken2string({Other, _}) when is_atom(Other) ->
-    io_lib:write(Other);
+    io_lib:write_atom(Other);
 yecctoken2string(Other) ->
-    io_lib:write(Other).
+    io_lib:format("~tp", [Other]).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
 
--file("./megaco_text_mini_parser.erl", 196).
+-file("megaco_text_mini_parser.erl", 185).
 
+-dialyzer({nowarn_function, yeccpars2/7}).
 yeccpars2(0=S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(S, Cat, Ss, Stack, T, Ts, Tzr);
 %% yeccpars2(1=S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -444,6 +434,7 @@ yeccpars2_1(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_1_(Stack),
  yeccpars2_4(4, Cat, [1 | Ss], NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_2/7}).
 yeccpars2_2(_S, '$end', _Ss, Stack, _T, _Ts, _Tzr) ->
  {ok, hd(Stack)};
 yeccpars2_2(_, _, _, _, T, _, _) ->
@@ -453,6 +444,7 @@ yeccpars2_3(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_3_(Stack),
  yeccgoto_optSep(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_4/7}).
 yeccpars2_4(S, 'AuditCapToken', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 9, Ss, Stack, T, Ts, Tzr);
 yeccpars2_4(S, 'AuditToken', Ss, Stack, T, Ts, Tzr) ->
@@ -600,6 +592,7 @@ yeccpars2_4(S, 'VersionToken', Ss, Stack, T, Ts, Tzr) ->
 yeccpars2_4(_, _, _, _, T, _, _) ->
  yeccerror(T).
 
+-dialyzer({nowarn_function, yeccpars2_5/7}).
 yeccpars2_5(S, 'EQUAL', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 6, Ss, Stack, T, Ts, Tzr);
 yeccpars2_5(_, _, _, _, T, _, _) ->
@@ -611,6 +604,7 @@ yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_7_(Stack),
  yeccgoto_safeToken(hd(Ss), Cat, Ss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_8/7}).
 yeccpars2_8(S, 'COLON', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 81, Ss, Stack, T, Ts, Tzr);
 yeccpars2_8(_, _, _, _, T, _, _) ->
@@ -834,6 +828,7 @@ yeccpars2_80(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 
 %% yeccpars2_81: see yeccpars2_4
 
+-dialyzer({nowarn_function, yeccpars2_82/7}).
 yeccpars2_82(S, 'COLON', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 83, Ss, Stack, T, Ts, Tzr);
 yeccpars2_82(_, _, _, _, T, _, _) ->
@@ -862,6 +857,7 @@ yeccpars2_86(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_86_(Stack),
  yeccpars2_89(89, Cat, [86 | Ss], NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_87/7}).
 yeccpars2_87(S, endOfMessage, Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 88, Ss, Stack, T, Ts, Tzr);
 yeccpars2_87(_, _, _, _, T, _, _) ->
@@ -1190,6 +1186,7 @@ yeccpars2_95(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_95_(Stack),
  yeccpars2_104(_S, Cat, [95 | Ss], NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_96/7}).
 yeccpars2_96(S, 'RSBRKT', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 99, Ss, Stack, T, Ts, Tzr);
 yeccpars2_96(_, _, _, _, T, _, _) ->
@@ -1379,6 +1376,7 @@ yeccpars2_104(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_104_(Stack),
  yeccgoto_daddr(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccpars2_105/7}).
 yeccpars2_105(S, 'GREATER', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 106, Ss, Stack, T, Ts, Tzr);
 yeccpars2_105(_, _, _, _, T, _, _) ->
@@ -1438,9 +1436,11 @@ yeccpars2_116(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_116_(Stack),
  yeccgoto_mId(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_authenticationHeader/7}).
 yeccgoto_authenticationHeader(1, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_4(4, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_daddr/7}).
 yeccgoto_daddr(94, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_96(96, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_daddr(95=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1448,27 +1448,35 @@ yeccgoto_daddr(95=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_daddr(97=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_98(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_deviceName/7}).
 yeccgoto_deviceName(89, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_113(113, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_domainAddress/7}).
 yeccgoto_domainAddress(86=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_92(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_domainName/7}).
 yeccgoto_domainName(86=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_91(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_mId/7}).
 yeccgoto_mId(86=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_90(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_megacoMessage/7}).
 yeccgoto_megacoMessage(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_2(2, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_message/7}).
 yeccgoto_message(4, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_87(87, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_mtpAddress/7}).
 yeccgoto_mtpAddress(89, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_112(112, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_optSep/7}).
 yeccgoto_optSep(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_1(1, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_optSep(84=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1484,14 +1492,17 @@ yeccgoto_optSep(112=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_optSep(113=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_115(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_pathName/7}).
 yeccgoto_pathName(89=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_111(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_portNumber/7}).
 yeccgoto_portNumber(100, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_102(102, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_portNumber(107, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_108(108, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_safeToken/7}).
 yeccgoto_safeToken(4, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_86(86, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_safeToken(6, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1515,6 +1526,7 @@ yeccgoto_safeToken(100=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_safeToken(107=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_101(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_safeToken2/7}).
 yeccgoto_safeToken2(4=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_safeToken2(6=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1539,21 +1551,21 @@ yeccgoto_safeToken2(107=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
 -compile({inline,yeccpars2_0_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_0_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_1_/1}).
--file("megaco_text_mini_parser.yrl", 238).
+-file("megaco_text_mini_parser.yrl", 239).
 yeccpars2_1_(__Stack0) ->
  [begin
    asn1_NOVALUE
   end | __Stack0].
 
 -compile({inline,yeccpars2_3_/1}).
--file("megaco_text_mini_parser.yrl", 232).
+-file("megaco_text_mini_parser.yrl", 233).
 yeccpars2_3_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1561,7 +1573,7 @@ yeccpars2_3_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_7_/1}).
--file("megaco_text_mini_parser.yrl", 269).
+-file("megaco_text_mini_parser.yrl", 270).
 yeccpars2_7_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1569,14 +1581,14 @@ yeccpars2_7_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_84_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_84_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_85_/1}).
--file("megaco_text_mini_parser.yrl", 237).
+-file("megaco_text_mini_parser.yrl", 238).
 yeccpars2_85_(__Stack0) ->
  [__8,__7,__6,__5,__4,__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1584,14 +1596,14 @@ yeccpars2_85_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_86_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_86_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_88_/1}).
--file("megaco_text_mini_parser.yrl", 230).
+-file("megaco_text_mini_parser.yrl", 231).
 yeccpars2_88_(__Stack0) ->
  [__4,__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1599,7 +1611,7 @@ yeccpars2_88_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_90_/1}).
--file("megaco_text_mini_parser.yrl", 240).
+-file("megaco_text_mini_parser.yrl", 241).
 yeccpars2_90_(__Stack0) ->
  [__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1607,28 +1619,28 @@ yeccpars2_90_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_94_/1}).
--file("megaco_text_mini_parser.yrl", 259).
+-file("megaco_text_mini_parser.yrl", 260).
 yeccpars2_94_(__Stack0) ->
  [begin
    [ ]
   end | __Stack0].
 
 -compile({inline,yeccpars2_95_/1}).
--file("megaco_text_mini_parser.yrl", 259).
+-file("megaco_text_mini_parser.yrl", 260).
 yeccpars2_95_(__Stack0) ->
  [begin
    [ ]
   end | __Stack0].
 
 -compile({inline,yeccpars2_97_/1}).
--file("megaco_text_mini_parser.yrl", 259).
+-file("megaco_text_mini_parser.yrl", 260).
 yeccpars2_97_(__Stack0) ->
  [begin
    [ ]
   end | __Stack0].
 
 -compile({inline,yeccpars2_98_/1}).
--file("megaco_text_mini_parser.yrl", 260).
+-file("megaco_text_mini_parser.yrl", 261).
 yeccpars2_98_(__Stack0) ->
  [__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1636,7 +1648,7 @@ yeccpars2_98_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_99_/1}).
--file("megaco_text_mini_parser.yrl", 257).
+-file("megaco_text_mini_parser.yrl", 258).
 yeccpars2_99_(__Stack0) ->
  [__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1644,7 +1656,7 @@ yeccpars2_99_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_101_/1}).
--file("megaco_text_mini_parser.yrl", 263).
+-file("megaco_text_mini_parser.yrl", 264).
 yeccpars2_101_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1652,14 +1664,14 @@ yeccpars2_101_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_102_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_102_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_103_/1}).
--file("megaco_text_mini_parser.yrl", 255).
+-file("megaco_text_mini_parser.yrl", 256).
 yeccpars2_103_(__Stack0) ->
  [__6,__5,__4,__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1667,7 +1679,7 @@ yeccpars2_103_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_104_/1}).
--file("megaco_text_mini_parser.yrl", 261).
+-file("megaco_text_mini_parser.yrl", 262).
 yeccpars2_104_(__Stack0) ->
  [__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1675,7 +1687,7 @@ yeccpars2_104_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_106_/1}).
--file("megaco_text_mini_parser.yrl", 250).
+-file("megaco_text_mini_parser.yrl", 251).
 yeccpars2_106_(__Stack0) ->
  [__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1683,14 +1695,14 @@ yeccpars2_106_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_108_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_108_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_109_/1}).
--file("megaco_text_mini_parser.yrl", 248).
+-file("megaco_text_mini_parser.yrl", 249).
 yeccpars2_109_(__Stack0) ->
  [__6,__5,__4,__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1698,7 +1710,7 @@ yeccpars2_109_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_110_/1}).
--file("megaco_text_mini_parser.yrl", 267).
+-file("megaco_text_mini_parser.yrl", 268).
 yeccpars2_110_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1706,7 +1718,7 @@ yeccpars2_110_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_111_/1}).
--file("megaco_text_mini_parser.yrl", 252).
+-file("megaco_text_mini_parser.yrl", 253).
 yeccpars2_111_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1714,21 +1726,21 @@ yeccpars2_111_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_112_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_112_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_113_/1}).
--file("megaco_text_mini_parser.yrl", 233).
+-file("megaco_text_mini_parser.yrl", 234).
 yeccpars2_113_(__Stack0) ->
  [begin
    no_sep
   end | __Stack0].
 
 -compile({inline,yeccpars2_114_/1}).
--file("megaco_text_mini_parser.yrl", 265).
+-file("megaco_text_mini_parser.yrl", 266).
 yeccpars2_114_(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
@@ -1736,7 +1748,7 @@ yeccpars2_114_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_115_/1}).
--file("megaco_text_mini_parser.yrl", 245).
+-file("megaco_text_mini_parser.yrl", 246).
 yeccpars2_115_(__Stack0) ->
  [__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1744,7 +1756,7 @@ yeccpars2_115_(__Stack0) ->
   end | __Stack].
 
 -compile({inline,yeccpars2_116_/1}).
--file("megaco_text_mini_parser.yrl", 244).
+-file("megaco_text_mini_parser.yrl", 245).
 yeccpars2_116_(__Stack0) ->
  [__3,__2,__1 | __Stack] = __Stack0,
  [begin
@@ -1752,4 +1764,4 @@ yeccpars2_116_(__Stack0) ->
   end | __Stack].
 
 
--file("megaco_text_mini_parser.yrl", 399).
+-file("megaco_text_mini_parser.yrl", 400).
